%DEMOL1ANALYSIS Demonstrates the usage of pdNCG solver with manual (user-
%defined) continuation. The tested instance is an L1-analysis signal 
%reconstruction problem:
%  
%  min tau*||W^Tx||_1 + 0.5*||Ax - b||^2, where W : R^p -> R^n.
%
% The problem instance that is used has been copied from 
% TFOCS package, url: http://cvxr.com/tfocs/download/.
%
% Copyright (c) 2014.Ioannis Dassios, Kimon Fountoulakis and Jacek Gondzio.

clear;

fprintf('Example L1-analysis \n \n');

% Load "Hallelujah" clip.
HANDEL = load('handel'); 
x_opt = HANDEL.y;        

% Play noiseless clip.
PLAY = input('Play the original music clip? (y/n) ','s');
if strcmpi(PLAY,'y') || strcmpi(PLAY,'yes'), sound(x_opt); end

N = length(x_opt);
% Zero padding.
k = nextpow2(N);
k = k-1;
x_opt = [x_opt(1:min(N,2^k)); zeros(2^k-N,1)];
N = 2^k;

% The measurement is the identity, i.e. denoising.
A = A_operator(@(x) x, @(x) x); 

% Noise function.
awgn = @(x,snr) x + 10^( (10*log10(sum(abs(x(:)).^2)/length(x(:))) - snr)/20 )*randn(size(x));

% Add noise.
snr     = 10;   % in dB
b       = A*x_opt;
x_noisy = awgn(x_opt,snr);
b_n     = A*x_noisy;

% Play noisy clip.
PLAY = input('Play the noisy music clip? (y/n) ','s');
if strcmpi(PLAY,'y') || strcmpi(PLAY,'yes'), sound( x_noisy ); end

% The following code regarding the analysis operator has been copied from
% Nesta algorithm, see http://www-stat.stanford.edu/~candes/nesta/#code.
%
% SETUP AN ANALYSIS OPERATOR 
%
% The PsiWFF and PsiTransposeWFF code is a Gabor frame
% (i.e. a short-time Fourier transform)
% written by Peter Stobbe
% 
% PsiWFF is the synthesis operator, and acts on coefficients
% PsiTransposeWFF is the analysis operator, and acts on signals
gMax        = 0;
% gMax        = -8; % 2.6e-1 er with -8
gLevels     = 1 - gMax;
tRedundancy = 1;
fRedundancy = 0;

gWindow = 'isine';
logN    = log2(N);
psi     = @(y) PsiWFF(y,gWindow,logN,logN-gLevels,logN+gMax,tRedundancy,fRedundancy);
psiT    = @(x) PsiTransposeWFF(x,gWindow,logN,logN-gLevels,logN+gMax,tRedundancy,fRedundancy);

W       = A_operator(@(q) psi(q), @(q) psiT(q));

% Options.
opts.n   = N;
opts.b   = b_n;
opts.A   = A;
opts.W   = W;
opts.tau = 0.17*norm(A'*(A*x_opt-b_n),'inf');
opts.problem = 'wavelet';

% Call pdNCG.
tt = cputime;
% Check input arguments.
opts = checkOptions(opts);

% initialize data.
order_mu  = max(-floor(log(abs(opts.mu))./log(10)),0);
order_tau = max(-floor(log(abs(opts.tau))./log(10)),0);
    
t_mu  = max(order_mu-1,0);
t_tau = max(order_tau-1,0);

if t_mu == 0
    mus_vector = ones(opts.cont_steps,1).*opts.mu;
else
    mus_vector = logspace(log10(0.1),log10(opts.mu),opts.cont_steps);
end

if t_tau == 0
    taus_vector = ones(opts.cont_steps,1).*opts.tau;
else
    taus_vector = logspace(log10(0.1),log10(opts.tau),opts.cont_steps);
end

opts.tol = 1.0e+2;

% Begin manual continuation.
for i=1:opts.cont_steps
    
    opts.mu  = mus_vector(i);  
    opts.tau = taus_vector(i);
    
    fprintf('%5s:  %d \n','Total continuation steps',opts.cont_steps);
    fprintf('%5s: %d \n','Current continuation step',i);   
    fprintf('%5s: %d \n','Current mu ',opts.mu);
    fprintf('%5s: %d \n','Current tau',opts.tau); 
    
    [x, dual, out] = pdNCG_W(opts);
    opts.x0 = x; 
    opts.y0 = dual;
    opts.tol = max(opts.tol/10,1.0e-2);
end
time_pdncg = cputime - tt;

% Play reconstructed clip.
PLAY = input('Play the recovered by pdNCG signal music clip? (y/n) ','s');
if strcmpi(PLAY,'y') || strcmpi(PLAY,'yes'), sound( x ); end

clear;
