function [x, y ,out] = pdNCGinterface(opts)
%PDNCGINTERFACE interface for pdNCG solver.
%
%  The pdNCG solver is a primal-dual Newton Conjugate Gradients algorithm.
%  The solver implements six formulation in total. Five of them are with 
%  pre-fixed objective function, while one of them is user-defined. 
%  
%  The solver implements:   
%  
%    min tau*||x||_1 + 0.5||Ax-b||_2^2,
%  
%    min tau*||x||_1 + logistic_regression(x),       
%  
%    min tau*||x||_1 + phi(x),
%  
%  where tau is a positive constant given by the user, and phi(x) is 
%  strongly-convex and twice differentiable. Additionally, some special 
%  formulations for signal/image reconstruction are implemented:
%  
%    min tau*||W^Tx||_1 + 0.5||Ax-b||_2^2,
%  
%    min tau*||x||_TV + 0.5||Ax-b||_2^2,
%  
%    min tau_W*||W^Tx||_1 + tau_TV*||x||_TV + 0.5||Ax-b||_2^2,
%  
%  where tau_W and tau_TV are positive constants given by the user, W and A 
%  are general matrices, ||x||_TV is the Total-Variation norm and b are the 
%  measurements.
%
%  *** The method works only on vector spaces. Therefore, in case of   ***
%  *** image processing problems the user will have to treat the image ***
%  *** as a vector. Examples are provided in demos folder.             ***
%
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%
% USAGE
% -----
%
% The standard call of pdNCG is
% 
%  [x, y, out] = pdNCGinterface(opts) 
% 
% Output
% 
% x are the primal variables, y are the dual variables and out is a MATLAB 
% structure which contains information about the solver:
% 
%  out.cput:        Total time required by the pdNCG solver.
%                   If MATLAB runs on multiple threads then wall-clock time
%                   is returned. Otherwise, CPU time is returned.
%  out.itr:         Number of pdNCG iterations.
%  out.message:     Termination status of the pdNCG solver. One of the 
%                   following two messages is returned:
%                   i)  'Converged'
%                   ii) 'Maximum iterations reached'
% 
% Input
% 
% The opts argument is mandatory and it is of type MATLAB structure.
% In what follows it is pointed out which fields of the options structure 
% are mandatory and addiotionally non-mandatory fields are explained.
% 
% Description of input arguments:
% 
% i) General options for solver (mandatory).
% 
%  opts.problem:      Three options:
%                     a) 'user' (phi(x) is user-defined) solves
%                         min tau*||x||_1 + phi(x).
% 
%                     b) 'ls' (Least-Squares) solves
%                         min tau*||x||_1 + 0.5||Ax-b||_2^2.
%  
%                     c) 'lg' (Logistic-Regression) solves
%                         min tau*||x||_1 + logistic_regression(x).
%  
%                     d) 'wavelet' (L1-analysis, signal reconstruction)
%                         solves min tau*||W^Tx||_1 + 0.5||Ax-b||_2^2.
%  
%                     e) 'tv' (Total-Variation, image reconstruction)
%                         solves min tau*||x||_TV + 0.5||Ax-b||_2^2.
% 
%                     f) 'tv_wavelet' (Combined Total-Variation with 
%                         L1-analysis; image reconstruction) solves
%                         min tau_W*||W^Tx||_1 + tau_TV*||x||_TV + 
%                                                          0.5||Ax-b||_2^2.                            
% 
% ii) Options mandatory for user-defined function phi(x).
% 
%  opts.n             Number of unknown variables
%  opts.tau:          Regularization parameter of l1-norm.
% 
%  opts.phi:          A function handle. It is called by pdNCG as phi(x).
%                     See demo demo_user.m in demos folder.
%                     
%  opts.gradient_phi: A function handle. It is called by pdNCG as 
%                     gradient_phi(x). See demo demo_user.m in demos 
%                     folder.
%                     
%  opts.hessian_phi:  A function handle. It is called by pdNCG as 
%                     hessian_phi(x). See demo demo_user.m in demos 
%                     folder.
% 
% iii) Options mandatory for Least-Squares.
% 
%  opts.n             Number of unknown variables.
%  opts.tau:          Regularization parameter of l1-norm.
% 
%  opts.A:            Matrix A for the Least-Squares fidelity term.
%  opts.b:            Data.
% 
% iv) Options mandatory for Logistic Regression.
% 
%  opts.n             Number of unknown variables.
%  opts.tau:          Regularization parameter of l1-norm.
% 
%  opts.X:            Matrix X stores the training samples.
%  opts.y:            Labels vector, must be -1 or 1.
% 
% v) Options mandatory for L1-analysis.
% 
%  opts.n             Number of unknown variables.
%  opts.tau:          Regularization parameter of l1-norm.
% 
%  opts.A:            Matrix A for the Least-Squares fidelity term.
%  opts.b:            Signal measurements.
% 
% vi) Options mandatory for Total-Variation.
% 
%  opts.n1            Number of vertical pixels of image.
%  opts.n2            Number of horizontal pixels of image.
%  opts.tau:          Regularization parameter of TV-norm.
% 
%  opts.A:            Matrix A for the Least-Squares fidelity term.
%  opts.b:            Image measurements.
% 
% vii) Options mandatory for Total-Variation combined with L1-analysis
% 
%  opts.n1            Number of vertical pixels of image.
%  opts.n2            Number of horizontal pixels of image.
%  opts.tau_TV:       Regularization parameter of TV-norm.
%  opts.tau_W:        Regularization parameter of L1-analysis regularizer.
% 
%  opts.A:            Matrix A for the Least-Squares fidelity term.
%  opts.b:            Samples b.
% 
% viii) Non-mandatory options for solvers.
% 
%  opts.mu:           Controls the quality of the approximation of the 
%                     l1-norm by pseudo-Huber. Default value is 1.0e-5.
%  opts.tol:          Optimality tolerance of pdNCG. The termination
%                     criterion is the rellative change of the primal
%                     variables. Default value: 1.0e-2.
%  opts.cont_steps    Number of continuation steps. Default value: the
%                     maximum order of magnitude between opts.mu and 
%                     opts.tau. For example, if opts.mu = 1.0e-5 and 
%                     opts.tau = 1.0e-2, then opts.cont_steps = 5. For 
%                     'tv_wavelet' problems the maximum order of magnitude 
%                     among opts.mu, opts.tau_W and opts.tau_TV is 
%                     calculated. For problems ls, lg and user the default 
%                     value is 1 (disabled).
%  opts.maxpdNCGitr:  Maximum pdNCG iterations.
%                     Default value: 50.
%  opts.tolpcg:       Optimality tolerance for PCG method.
%                     Default value: 1.0e-1.
%  opts.mxiterpcg:    Maximum iterations of PCG method.
%                     Default value: 10000.
%  opts.maxstepsize:  Maximum stepsize for primal directions.
%                     Default value 1.0.
%  opts.reg:          Regularization constant of the second order 
%                     derivative. Default value: 1.0e-2. For Least-Squares
%                     problems (ls, wavelet, tv, tv_wavelet) that A^TA=I 
%                     the default value is 1.0e-16.
%  opts.x0:           The initial guess of the algorithm. 
%                     Default value: 0.
%  opts.y0:           The initial guess for dual variables, for problems ls,
%                     lg, user, tv and wavelet.
%  opts.y_TV0:        The initial guess for dual variables of the TV-norm, 
%                     for problem tv_wavelet.
%  opts.y_W0:         The initial guess for dual variables of the l1-norm, 
%                     for problem tv_wavelet.
%  opts.max_itr_bt    Maximum iterations for backtracking linesearch.
%                     Default: 10.
%  opts.c2            Parameter of backtracking linesearch that constrols
%                     the sufficient decrease.
%                     Default: 1.0e-3.
%  opts.c3            Paarameter of backtracking linesearch that constrols
%                     the decrease of the step-size at every iteration of 
%                     linesearch.
%                     Default = 0.9.
%  opts.rankupdates   Rank updates for the L-BFGS preconditioner.
%                     Default value: 6. This applies to problems ls, lg,
%                     user.
% opts.whic_prec      For TV problems only. Choose between two ways of 
%                     applying the peconditioner.
%                     1 for exact solution of linear systems with the 
%                     preconditioner. 0 for approximate solution of linear 
%                     systems using CG.
%                     Default: 1
% opts.prec_itrs      For TV problems only. If the parameter 
%                     opts.which_prec has been set to 0, then this  
%                     parameter controls the number of iterations of CG,  
%                     which is used to solve linear systems approximately  
%                     with the preconditioner. There is a tradeoff here.  
%                     The more the iterations, the better the clustering of  
%                     the eigenvalues of the preconditioned systems, 
%                     but this increases the computational cost of the 
%                     application of the preconditioner.
%                     Default: 15
%  opts.prec_tol      Tolerance for enabling preconditioning for
%                     Total-Variation and L1-analysis. For example, if
%                     opts.mu <= opts.prec_tol, then preconditioning is
%                     enabled.
%                     Default: 1.0e-4;
%  opts.verbose:      0 for no printing info, 1 for basic printing info
%                     and 2 for detailed printing info.
%                     Default value: 2.
%  opts.user_fcn:     This is a function handle, which if it is defined, it 
%                     is called at every iteration of pdNCG. It returns to 
%                     the user information about pdNCG such that the user 
%                     can experiment with the algorithm. 
%                     This function is called from pdNCG as 
%                     user_fcn(opts,primal,dual,f,g,H,P,d) at every 
%                     iteration.
%                     opts: is all parameters of pdNCG. 
%                     primal: are the current primal variables.
%                     dual: are the current dual variables.
%                     f: is a function handle for the objective function.
%                     g: is the current gradient. 
%                     H: is an approximation to the Hessian of the 
%                        objective function with which linear systems are
%                        solved.
%                     P: is a preconditioner for H.
%                     d: is the current direction.
%                     An example on how to use this optional argument is 
%                     given in the demos folder in file demo_user_fcn.m.
%  opts.user_stop:    This is a function handle, which if it is defined, it 
%                     is called at every iteration of pdNCG. It passes to 
%                     the user information such that the user can
%                     define its own termination criteria. The user defined
%                     criteria replace the default termination criteria.
%                     This function is called from pdNCG as 
%                     user_stop(opts,primal,dual,f,g,H,d) at every 
%                     iteration.
%                     opts: is all parameters of pdNCG. 
%                     primal: are the current primal variables.
%                     dual: are the current dual variables.
%                     f: is a function handle for the objective function.
%                     g: is the current gradient. 
%                     H: is an approximation to the Hessian of the 
%                        objective function, with which linear systems are
%                        solved.
%                     d: is the current direction.
%                     An example on to use this optional argument is given 
%                     in the demos folder in file demo_user_stop.m.
% 
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
% Copyright (c) 2014.Ioannis Dassios, Kimon Fountoulakis and Jacek Gondzio.

if ~isfield(opts, {'problem'})
   error('Possible options for opts.problem: a) user, b) ls, c) lg.'); 
end

if strcmp(opts.problem,'ls')
  
    % Initialize parameters.
    opts    = checkOptions(opts);
    % Call continuation framework with the solver as an argument.
    [x, y ,out] = cont_framework(opts,@pdNCG_ls);
    
elseif strcmp(opts.problem,'lg')
    
    % Initialize parameters.
    opts    = checkOptions(opts);
    % Call continuation framework with the solver as an argument.
    [x, y ,out] = cont_framework(opts,@pdNCG_lg);
    
elseif strcmp(opts.problem,'wavelet')
    
    % Initialize parameters.
    opts    = checkOptions(opts);
    % Call continuation framework with the solver as an argument.
    [x, y ,out] = cont_framework(opts,@pdNCG_W);
    
elseif strcmp(opts.problem,'tv')
    
    % Initialize parameters.
    opts    = checkOptions(opts);
    % Call continuation framework with the solver as an argument.
    [x, y ,out] = cont_framework(opts,@pdNCG_TV);
    
elseif strcmp(opts.problem,'tv_wavelet')
     
    % Initialize parameters.
    opts    = checkOptions(opts);
    % Call continuation framework with the solver as an argument.
    [x, y ,out] = cont_framework_tv_w(opts,@pdNCG_TV_W);
    
elseif strcmp(opts.problem,'user')
    
    % Initialize parameters.
    opts    = checkOptions(opts);
    % Call continuation framework with the solver as an argument.
    [x, y ,out] = cont_framework(opts,@pdNCG_user);
    
else
    error('Possible options for opts.problem: a) user, b) ls, c) lg.');     
end