function [x, y, out] = pdNCG_W(opts)
%PDNCG pdNCG solver for the l1-regularized problem
%
% min tau*||W^Tx||_1 + 0.5||Ax-b||_2^2,
%
% Copyright (c) 2014.  Ioannis Dassios, Kimon Fountoulakis and Jacek Gondzio.

%% Initialize output.
out.itr  = 1; % pdNCG iterations.

%% Check if A'*A = I, i.e. denoising.
tmp = rand(opts.n,1);
if norm(opts.A'*(opts.A*tmp)-tmp,1)/norm(tmp,1) < 1e-15;
    opts.reg = 1.0e-16;
    rho      = 1;
else
    rho = 0.5;
end
clear tmp;

%% Initialize data.
stagnation = 0;

nwtn = zeros(opts.n,1);

x    = opts.x0;
Wtx  = opts.W'*x;
D    = 1./sqrt(opts.mu^2 + Wtx.^2);
DWtx = D.*Wtx;

% Initialize dual variables.
y = opts.y0;

% Calculate the approximate Hessian of pseudo-Huber function.
scale = D.*(1 - DWtx.*y);
H     = @(q) opts.W*(scale.*(opts.W'*q));

% Calculate gradient at x0.
g_ls = opts.A.'*(opts.A*x-opts.b);
g_ph = opts.W*DWtx;
g    = opts.tau.*g_ph + g_ls;

% Fidelity and regularization functions.
f_ls    = @(q) 0.5*norm(opts.A*q-opts.b)^2;
f_huber = @(q) opts.tau*sum(sqrt(opts.mu^2 + (opts.W'*q).^2) - opts.mu);
     
%% Print
if opts.verbose >= 1
    fprintf('%5s | %5s    | %5s | %5s |  %5s  \n',...
        'pdNCG itr', 'a', '|d|_x/|x|','PCG itr', 'PCG rs');
end

%% Start Newton conjugate gradients.
while 1        
    % Define regularized second-order derivative.
    Hessian = @(q) opts.tau.*H(q) + opts.A'*(opts.A*q) + opts.reg.*q;

    % Define preconditioner function.
    if opts.mu <= opts.prec_tol;
        Preconditioner = @(q) prec_W(opts,q,H,rho,[],opts.prec_itrs);
    else 
        Preconditioner = @(q) q;        
    end

    % Call user function.
    if isfield(opts, {'user_fcn'}) 
        opts.user_fcn(opts,x,y,@(q) f_huber(q) + f_ls(q),g,Hessian,Preconditioner,nwtn);
    end    
    
    % Calculate Newton direction. 
    [nwtn, ~, rlrs, pcgi] = pcg(Hessian,-g,opts.tolpcg,opts.mxitrpcg,...
                                Preconditioner);       
    
    normN_scaled = -(nwtn'*g + opts.reg*norm(nwtn)^2)/norm(x);    
        
    Wtnwtn = opts.W'*nwtn;
    dy = scale.*Wtnwtn - (y - DWtx);
        
    y_tilde = y + dy;
    y = sign(y_tilde).*min(abs(y_tilde),1);
        
    % Calculate stepsize and update variables.        
    [x, a] = simple_bt(x, nwtn, f_ls, f_huber, normN_scaled, opts);
        
    % Printing info.
    if opts.verbose >= 1
        fprintf('  %4d    | %1.2e |  %8s | %4d    | %1.2e \n',...
            out.itr, a, num2str(normN_scaled,'%1.2e\n'), pcgi, rlrs);
    end
        
    % Calculate hessian of pseudo-Huber function.
    Wtx   = opts.W'*x;
    D     = 1./sqrt(opts.mu^2 + Wtx.^2);
    DWtx  = D.*Wtx;
    scale = D.*(1 - DWtx.*y);
    H     = @(q) opts.W*(scale.*(opts.W'*q));
        
    % Calculate first-order derivative of perturbed problem.
    g_ls  = opts.A.'*(opts.A*x-opts.b);
    g_ph  = opts.W*DWtx;
    g     = opts.tau.*g_ph + g_ls;
    
    if isfield(opts, {'user_stop'}) 
        if opts.user_stop(opts,x,y,@(q) f_huber(q) + f_ls(q),g,Hessian,nwtn)
            out.message = 'Converged';
            break;
        end
    else
        if normN_scaled <= opts.tol
            out.message = 'Converged';
            break; 
        end         
    end
    
    % Maximum iterations reached.
    if out.itr > opts.maxpdNCGitr
        stagnation = 1;
    end
    
    if stagnation == 1
       out.message = 'Maximum iterations of pdNCG reached';
       fprintf('Maximum iterations of pdNCG reached, termination.\n'); 
       fprintf('Possible fixes: \n');
       fprintf('a) increase opts.maxpdNCGitr. Performs more pdNCG iterations.\n');
       fprintf('b) decrease opts.tolpcg. Calculates better directions. \n');
       fprintf('c) increase opts.mxitrpcg. \n');
       break;
    end 
   
    % Update iteration counter.
    out.itr = out.itr + 1;
end
%% End of primal-dual Newton conjugate gradients.

% Call user function.
if isfield(opts, {'user_fcn'}) 
    opts.user_fcn(opts,x,y,@(q) f_huber(q) + f_ls(q),g,Hessian,Preconditioner,nwtn);
end   