function [x, y, out] = pdNCG_user(opts)
%PDNCG pdNCG solver for the l1-regularized problem
%
% min tau*||x||_1 + phi(x),
%
% where phi(x) is a user-defined strongly convex function. 
%
% Copyright (c) 2014.  Kimon Fountoulakis and Jacek Gondzio.

%% Initialize output.
out.itr  = 1; % pdNCG iterations.

%% Initialize data.
stagnation = 0;

nwtn = zeros(opts.n,1);

x = opts.x0;

D  = 1./sqrt(opts.mu^2 + x.^2);
Dx = D.*x;

y = opts.y0;

% Calculate the approximate Hessian of pseudo-Huber function.
H  = D.*(1 - Dx.*y);
        
tauH = opts.tau.*H;

% Calculate gradient at x0.
g = opts.tau.*Dx + opts.gradient_phi(x);

% Fidelity and regularization functions.
f_phi   = @(q) opts.phi(q);
f_huber = @(q) opts.tau*sum(sqrt(opts.mu^2 + q.^2) - opts.mu);
    
%% Print
if opts.verbose >= 1
    fprintf('%5s | %5s    | %5s    | %5s    | %5s | %5s |  %5s  \n',...
        'pdNCG itr', 'a', 'tau', 'mu', '|d|_x/|x|','PCG itr', 'PCG rs');
end

%% Start Newton conjugate gradients.
while 1        
    % Define regularized second-order derivative.
    Hessian = @(q) tauH.*q + opts.hessian_phi(x,q) + opts.reg*q;

	% Use L-BFGS Preconditioner
	if out.itr == 1
        % Initialize variables for preconditioner.
		S = zeros(opts.n,opts.rankupdates);
		Y = zeros(opts.n,opts.rankupdates);
		YS = zeros(opts.rankupdates,1);
		lbfgs_start = 1;
		lbfgs_end = 0;
		Hdiag = 1;      
        
        Preconditioner = @(q) q;
    else
        % Update data for preconditioner.
		[S,Y,YS,lbfgs_start,lbfgs_end,Hdiag] = lbfgsAdd(g-g_old,a*nwtn,...
            S,Y,YS,lbfgs_start,lbfgs_end,Hdiag,1);
        
        % Define preconditioner.
		Preconditioner = @(q) lbfgsProdC(q,S,Y,YS,int32(lbfgs_start),...
            int32(lbfgs_end),Hdiag);        
    end
    
    % Call user function.
    if isfield(opts, {'user_fcn'}) 
        opts.user_fcn(opts,x,y,@(q) f_huber(q) + f_ls(q),g,Hessian,Preconditioner,nwtn);
    end  
    
    % Calculate Newton direction.  
    [nwtn, ~, rlrs, pcgi] = pcg(Hessian,-g,opts.tolpcg,opts.mxitrpcg,...
                                Preconditioner);    
    
    normN_scaled = -(nwtn'*g + opts.reg*norm(nwtn)^2)/norm(x);    
        
    dy = H.*nwtn - (y - Dx);
        
    y_tilde = y + dy;
    y = sign(y_tilde).*min(abs(y_tilde),1);
        
    % Calculate stepsize and update variables.
    [x, a] = simple_bt(x, nwtn, f_phi, f_huber, normN_scaled, opts);
        
    % Printing info.
    if opts.verbose >= 1
        fprintf('  %4d    | %1.2e | %1.2e | %1.2e |  %8s | %4d    | %1.2e \n',...
            out.itr, a, opts.tau, opts.mu, num2str(normN_scaled,'%1.2e\n'), pcgi, rlrs);
    end
        
    % Calculate hessian of pseudo-Huber function.
    D  = 1./sqrt(opts.mu^2 + x.^2);
    Dx = D.*x;
    H  = D.*(1 - Dx.*y);
        
    tauH = opts.tau.*H;
        
    % Calculate first-order derivative of perturbed problem.
    g_old = g;
    g     = opts.tau.*Dx + opts.gradient_phi(x);
    
    if isfield(opts, {'user_stop'}) 
        if opts.user_stop(opts,x,y,@(q) f_huber(q) + f_ls(q),g,Hessian,nwtn)
            out.message = 'Converged';
            break;
        end
    else
        if normN_scaled <= opts.tol
            out.message = 'Converged';
            break; 
        end         
    end
    
    % Maximum iterations reached.
    if out.itr > opts.maxpdNCGitr
        stagnation = 1;
    end
    
    if stagnation == 1
       out.message = 'Maximum iterations of pdNCG reached';
       fprintf('Maximum iterations of pdNCG reached, termination.\n'); 
       fprintf('Possible fixes: \n');
       fprintf('a) increase opts.maxpdNCGitr. Performs more pdNCG iterations.\n');
       fprintf('b) decrease opts.tolpcg. Calculates better directions. \n');
       fprintf('c) increase opts.mxitrpcg. \n');
       break;
    end 
    
    % Update iteration counter.
    out.itr = out.itr + 1;
end
%% End of primal-dual Newton conjugate gradients.

% Call user function.
if isfield(opts, {'user_fcn'}) 
    opts.user_fcn(opts,x,y,@(q) f_huber(q) + f_ls(q),g,Hessian,Preconditioner,nwtn);
end  