function opts = checkOptions_TV_W(opts)
%CHECKOPTION Checks the input arguments for pdNCG_TV_W.
%
% Copyright (c) 2014.Ioannis Dassios, Kimon Fountoulakis and Jacek Gondzio.

%%
if ~isfield(opts, {'n1'})  
    error('The number vertical pixels n1 of the image is not provided.');
end    
if ~isfield(opts, {'n2'})  
    error('The number horizontal pixels n2 of the image n2 is not provided.');
end   
opts.n = opts.n1*opts.n2;

%%
if ~isfield(opts, {'A'})
    error('The measurement matrix opts.A is required.');
else
    if ~isa(opts.A, 'A_operator') && ~(sum(size(opts.A) > 1) == 2)
        error('The argument opts.A must be a MATLAB matrix or of type "A_operator" which operates on vectors.');
    end 
end
 
%%
if ~isfield(opts, {'W'})
    error('The matrix opts.W is required.');
else
    if ~isa(opts.W, 'A_operator') && ~(sum(size(opts.W) > 1) == 2)
        error('The argument opts.W must be a MATLAB matrix or of type "A_operator" which operates on vectors.');
    end
end
  
%%
if ~isfield(opts, {'b'})    
    error('Measurements vector b for the Least-Squares fidelity term is required.');
end

%%
sizes_b = size(opts.b);
if sizes_b(1) > 1 && sizes_b(2) > 1
    error('Measurements vector b should be a column vector.');
end
if sizes_b(2) > 1
    opts.b = opts.b';
end      

%%
if ~isfield(opts, {'tau_TV'}) 
    error('The regularization parameter tau_TV is required.');  
else
    if opts.tau_TV <= 0
        error('The regularization parameter opts.tau_TV must be greater than 0.');
    end
end
if ~isfield(opts, {'tau_W'}) 
    error('The regularization parameter tau_W is required.');  
else
    if opts.tau_W <= 0
        error('The regularization parameter opts.tau_W must be greater than 0.');
    end
end

%%
if ~isfield(opts, {'cont_steps'})  
    order_tau_TV = floor(log(abs(opts.tau_TV))./log(10));
    order_tau_W  = floor(log(abs(opts.tau_W))./log(10));
    order_mu     = floor(log(abs(opts.mu))./log(10));
    order = min([order_tau_TV,order_tau_W,order_mu]);
    if order < -1
        opts.cont_steps = -order;  
    else
        opts.cont_steps = 1;  
    end         
else
    if opts.cont_steps <= 0
        error('Continuation steps must be greater than 0.'); 
    end
end

%%
if ~isfield(opts, {'x0'})  
    opts.x0 = zeros(opts.n,1);
else
    size_x0 = size(opts.x0);
    if size_x0(1) > 1 && size_x0(2) > 1
        error('The argument opts.x0 should be a vector.');
    end
    if size_x0(2) > 1
        opts.x0 = opts.x0';
    end
end

%% Built TV matrices.
e          = ones(max(opts.n1,opts.n2),1);
e2         = e;
e2(opts.n2:end) = 0;
J          = spdiags([-e2,e], 0:1,opts.n2,opts.n2);
I          = sparse(1:opts.n1,1:opts.n1,ones(opts.n1,1));
opts.Wt_TV_re      = kron(J,I);  % horizontal differences, sparse matrix
            
e2         = e;
e2(opts.n1:end) = 0;
J          = spdiags([-e2,e], 0:1,opts.n1,opts.n1);
I          = sparse(1:opts.n2,1:opts.n2,ones(opts.n2,1));
opts.Wt_TV_im      = kron(I,J);  % vertical differences, sparse matrix

%%
if ~isfield(opts, {'y_TV0'})  
    Wt_re_x = opts.Wt_TV_re*opts.x0;
    Wt_im_x = opts.Wt_TV_im*opts.x0;
    Wtx = Wt_re_x +1i*Wt_im_x;
    
    y_tilde = Wtx;
    y_bar   = min(1./abs(y_tilde),1).*y_tilde;
    y_TV_re    = real(y_bar);
    y_TV_im    = imag(y_bar);
    opts.y_TV0 = y_TV_re + 1i*y_TV_im;
else
    size_y0 = size(opts.y_TV0);
    if size_y0(1) > 1 && size_y0(2) > 1
        error('The argument opts.y_TV0 should be a vector.');
    end
    if size_y0(2) > 1
        opts.y_TV0 = opts.y_TV0';
    end
    if size_y0(1) ~= opts.n
        error('The length of vector opts.y0 must be equal to opts.n1 times opts.n2.');
    end
end

%%
if ~isfield(opts, {'y_W0'})      
    Wtx  = opts.W'*opts.x0;
    opts.y_W0 = sign(Wtx).*min(abs(Wtx),1);
else    
    size_y0 = size(opts.y_W0);
    if size_y0(1) > 1 && size_y0(2) > 1
        error('The argument opts.y_W0 should be a vector.');
    end
    if size_y0(2) > 1
        opts.y_W0 = opts.y_W0';
    end
    if size_y0(1) ~= length(opts.W'*opts.x0)
        error('The length of vector opts.y_W0 must be equal to number of rows of matrix opts.W.');
    end
end

%%
if isfield(opts, {'user_stop'}) 
    if strcmpi(class(opts.user_stop), 'function_handle')
        numInputs  = nargin(opts.user_stop);
        if numInputs ~= 8
            error('The optional function user_stop must have 8 input arguments.'); 
        end
        out_user_stop = opts.user_stop(randn(opts.n,1),randn(opts.n,1));
        if ~islogical(out_user_stop)
            error('The optional function user_stop must have 1 logical output.'); 
        end
    else
        error('The optional function user_stop must be of type function_handle.'); 
    end
end

%%
if isfield(opts, {'user_fcn'}) 
    if strcmpi(class(opts.user_fcn), 'function_handle')
        numInputs  = nargin(opts.user_fcn);
        numOutputs = nargout(opts.user_fcn);
        if numInputs ~= 9 || numOutputs ~= -1
            error('The optional function user_fcn must have 9 input arguments and no output.'); 
        end
    else
        error('The optional function user_fcn must be of type function_handle.'); 
    end
end